/*
 * Copyright (c) 2017-2021, Arm Limited. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef CACTUS_TESTS_H
#define CACTUS_TESTS_H

#include <spm_common.h>

/*
 * Test functions
 */

/*
 * Alter SIMD vectors to check saving of the context while switching between
 * the normal world and the secure world.
 */
void fill_simd_vectors(void);

/*
 * Test to FFA interfaces.
 */
void ffa_memory_management_test(struct mailbox_buffers *mb, ffa_vm_id_t vm_id,
				ffa_vm_id_t sender, uint32_t mem_func,
				uint64_t handle);

void ffa_tests(struct mailbox_buffers *mb);

/*
 * Test other things like the version number returned by SPM.
 */
void misc_tests(void);

/*
 * The Arm TF is responsible for setting up system registers on behalf of the
 * Secure Partition. For example, TF is supposed to allow Secure Partitions to
 * perform cache maintenance operations (by setting the SCTLR_EL1.UCI bit).
 *
 * This function attempts to verify that we indeed have access to these system
 * features from S-EL0. These tests report their results on the UART. They do
 * not recover from a failure : when an error is encountered they will most
 * likely trigger an exception into S-EL1.
 */
void system_setup_tests(void);

/*
 * Exercise the SP_MEMORY_ATTRIBUTES_SET_AARCH64 SMC interface. A variety of
 * valid and invalid requests to change memory attributes are tested.
 *
 * These tests report their results on the UART. They do not recover from a
 * failure : when an error is encountered they endlessly loop.
 */
void mem_attr_changes_tests(void);

#endif /* CACTUS_TESTS_H */
